// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package impl

import (
	"context"
	"cp-organization-management/impl/v1/db"
	"cp-organization-management/impl/v1/log"
	"cp-organization-management/impl/v1/rbac"
	"cp-organization-management/impl/v1/user"
	"cp-organization-management/impl/v1/zone"
	"cp-organization-management/pb"
	pb_v1 "cp-organization-management/pb/v1"

	"github.com/jaryhe/gopkgs/tasker/rpctasker"
	"google.golang.org/grpc"
)

// 具体实现
type Rcvr struct {
}

func Register(s *grpc.Server) {
	pb.RegisterOrganizationServer(s, &Rcvr{})
}

// 初始化机构库
func (c *Rcvr) InitOrganizationDb(ctx context.Context, req *pb_v1.InitOrganizationDbRequest) (reply *pb_v1.InitOrganizationDbReply, err error) {
	t1 := func() error {
		reply, err = db.InitOrganizationDb(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 登录
func (c *Rcvr) ManagementLogin(ctx context.Context, req *pb_v1.ManagementLoginRequest) (reply *pb_v1.ManagementLoginReply, err error) {
	t1 := func() error {
		reply, err = user.ManagementLogin(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 创建账户
func (c *Rcvr) CreateManagementUser(ctx context.Context, req *pb_v1.CreateManagementUserRequest) (reply *pb_v1.CreateManagementUserReply, err error) {
	t1 := func() error {
		reply, err = user.CreateManagementUser(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 创建超级管理员，系统端调用
func (c *Rcvr) ManagementSuperUserList(ctx context.Context, req *pb_v1.ManagementSuperUserListRequest) (reply *pb_v1.ManagementSuperUserListReply, err error) {
	t1 := func() error {
		reply, err = user.ManagementSuperUserList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 修改超级管理员，系统端调用
func (c *Rcvr) ManagementSuperUserUpdate(ctx context.Context, req *pb_v1.ManagementSuperUserUpdateRequest) (reply *pb_v1.ManagementSuperUserUpdateReply, err error) {
	t1 := func() error {
		reply, err = user.ManagementSuperUserUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 修改超级管理员密码，系统端调用，暂时废弃
func (c *Rcvr) ManagementSuperUserResetPassword(ctx context.Context, req *pb_v1.ManagementSuperUserResetPasswordRequest) (reply *pb_v1.ManagementSuperUserResetPasswordReply, err error) {
	t1 := func() error {
		reply, err = user.ManagementSuperUserResetPassword(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 所有权限节点
func (c *Rcvr) RbacNodeList(ctx context.Context, req *pb_v1.RbacNodeListRequest) (reply *pb_v1.RbacNodeListReply, err error) {
	t1 := func() error {
		reply, err = rbac.RbacNodeList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 角色列表
func (c *Rcvr) RbacGroupList(ctx context.Context, req *pb_v1.RbacGroupListRequest) (reply *pb_v1.RbacGroupListReply, err error) {
	t1 := func() error {
		reply, err = rbac.RbacGroupList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 角色或用户下的权限节点
func (c *Rcvr) RbacNodeListByGroupOrUser(ctx context.Context, req *pb_v1.RbacNodeListByGroupOrUserRequest) (reply *pb_v1.RbacNodeListByGroupOrUserReply, err error) {
	t1 := func() error {
		reply, err = rbac.RbacNodeListByGroupOrUser(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 添加角色
func (c *Rcvr) RbacGroupAdd(ctx context.Context, req *pb_v1.RbacGroupAddRequest) (reply *pb_v1.RbacGroupAddReply, err error) {
	t1 := func() error {
		reply, err = rbac.RbacGroupAdd(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 修改角色
func (c *Rcvr) RbacGroupUpdate(ctx context.Context, req *pb_v1.RbacGroupUpdateRequest) (reply *pb_v1.RbacGroupUpdateReply, err error) {
	t1 := func() error {
		reply, err = rbac.RbacGroupUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 删除角色
func (c *Rcvr) RbacGroupDel(ctx context.Context, req *pb_v1.RbacGroupDelRequest) (reply *pb_v1.RbacGroupDelReply, err error) {
	t1 := func() error {
		reply, err = rbac.RbacGroupDel(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 添加区域
func (c *Rcvr) ZoneAdd(ctx context.Context, req *pb_v1.ZoneAddRequest) (reply *pb_v1.ZoneAddReply, err error) {
	t1 := func() error {
		reply, err = zone.ZoneAdd(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 删除区域
func (c *Rcvr) ZoneDel(ctx context.Context, req *pb_v1.ZoneDelRequest) (reply *pb_v1.ZoneDelReply, err error) {
	t1 := func() error {
		reply, err = zone.ZoneDel(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 修改区域
func (c *Rcvr) ZoneUpdate(ctx context.Context, req *pb_v1.ZoneUpdateRequest) (reply *pb_v1.ZoneUpdateReply, err error) {
	t1 := func() error {
		reply, err = zone.ZoneUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 区域列表
func (c *Rcvr) ZoneList(ctx context.Context, req *pb_v1.ZoneListRequest) (reply *pb_v1.ZoneListReply, err error) {
	t1 := func() error {
		reply, err = zone.ZoneList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}


func (c *Rcvr) UserUpdate(ctx context.Context, req *pb_v1.UserUpdateRequest) (reply *pb_v1.UserUpdateReply, err error) {
	t1 := func() error {
		reply, err = user.UserUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) UserDel(ctx context.Context, req *pb_v1.UserDelRequest) (reply *pb_v1.UserDelReply, err error) {
	t1 := func() error {
		reply, err = user.UserDel(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) UserList(ctx context.Context, req *pb_v1.UserListRequest) (reply *pb_v1.UserListReply, err error) {
	t1 := func() error {
		reply, err = user.UserList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) UserInfo(ctx context.Context, req *pb_v1.UserInfoRequest) (reply *pb_v1.UserInfoReply, err error) {
	t1 := func() error {
		reply, err = user.UserInfo(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) ManagementLogAdd(ctx context.Context, req *pb_v1.ManagementLogAddRequest) (reply *pb_v1.ManagementLogAddReply, err error) {
	t1 := func() error {
		reply, err = log.ManagementLogAdd(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) ManagementLogList(ctx context.Context, req *pb_v1.ManagementLogListRequest) (reply *pb_v1.ManagementLogListReply, err error) {
	t1 := func() error {
		reply, err = log.ManagementLogList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

