// Copyright 2019 autocareai.com. All rights reserved.
// Use of this source code is governed by autocareai.com.

package user

import (
	"context"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)


func getOrigin(id int64, organizationCode, dbname string ) (*pb_v1.ManagementSuperUserUpdateRequest, *model.RbacUser, error) {
	p := model.NewRbacUser(dbname)

	where := map[string]interface{}{
		"id":id,
	}
	err := p.Find(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil, errors.ErrRecordNotFound
		}
		return nil, nil, errors.DataBaseError
	}
	ret := &pb_v1.ManagementSuperUserUpdateRequest{}
	ret.Username = p.Username
	ret.Id = id
	ret.OrganizationCode = organizationCode
	ret.Phone = p.Phone
	ret.Email = p.Email
	ret.Password = "******"
	return ret, p, nil
}

func ManagementSuperUserUpdate(ctx context.Context, req *pb_v1.ManagementSuperUserUpdateRequest) (reply *pb_v1.ManagementSuperUserUpdateReply, err error) {
	reply = &pb_v1.ManagementSuperUserUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if  req.OrganizationCode == "" || req.Username == "" || req.Id == 0{
		return reply, status.Error(10003, "参数错误")
	}


	dbname := utils.GetDbName(req.OrganizationCode)



	p := model.NewRbacUser(dbname)

	origin, user, err := getOrigin(req.Id, req.OrganizationCode, dbname)
	if err != nil {
		return nil, err
	}

	if origin.Username != req.Username {
		user := model.NewRbacUser(dbname)
		where := map[string]interface{}{
			"username":req.Username,
		}
		err := user.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound {
			return nil, errors.DataBaseError
		}
		if user.Id > 0 {
			return nil, errors.UserExist
		}
	}



	where := map[string]interface{}{
		"id":req.Id,
	}
	values := map[string]interface{}{
		"username":req.Username,
		"phone":req.Phone,
		"email":req.Email,
	}
	if req.Password != "" {
		values["password"] = req.Password
	}
	db := database.DB().Begin()
	rows, err := p.UpdateWithAffectRows(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if rows == 0 {
		db.Rollback()
		return nil, errors.ErrRecordNotFound
	}

	user.Email = req.Email
	user.Phone = req.Phone
	user.Username = req.Username
	if err := common.PutUserBaseInfo(user, dbname); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	reply.Origin = origin

	return reply, nil
}
