package zone

import (
	"context"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"strings"
)


func getChild(pcode string, m map[string][]*pb_v1.ZoneItem) (ret []*pb_v1.ZoneItem) {
	if _, ok := m[pcode]; !ok {
		return ret
	}
	ret = append(ret, m[pcode]...)
	for k, v := range ret {
		ret[k].Childs = getChild(v.ZoneCode, m)
	}
	return ret
}

func ZoneTree(array []model.Zone) (ret []*pb_v1.ZoneItem) {
	m := map[string][]*pb_v1.ZoneItem{}
	min := ""
	for _, v := range array {
		item := pb_v1.ZoneItem{
			ZoneCode:v.ZoneCode,
			ParentZoneCode:v.ParentZoneCode,
			ZoneName:v.ZoneName,
		}
		m[v.ParentZoneCode] = append(m[v.ParentZoneCode], &item)
		if min == "" {
			min = v.ParentZoneCode
			continue
		}
		if len(v.ParentZoneCode) < len(min) {
			min = v.ParentZoneCode
			continue
		}
		if strings.Compare(min, v.ParentZoneCode) > 0 {
			min = v.ParentZoneCode
		}
	}


	return getChild(min, m)
}

func ZoneList(ctx context.Context, req *pb_v1.ZoneListRequest) (reply *pb_v1.ZoneListReply, err error) {
	reply = &pb_v1.ZoneListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.OrganizationCode == ""{
		return nil, errors.ParamsError
	}

	dbname := utils.GetDbName(req.OrganizationCode)
	p := model.NewZone(dbname)
	where := map[string]interface{}{
		"zone_code in" : "",
	}
	list := []model.Zone{}
	super, userZones, err := common.GetUserZone(req.Uid, dbname)
	if err != nil {
		return nil, err
	}
	if super {
		where = nil
	} else if len(userZones) == 0 {
		return reply, nil
	} else {
		zones := []string{}
		for k, _ := range userZones {
			zones = append(zones, k)
		}
		where["zone_code in"] = zones
	}

	err = p.QueryAllNew(database.DB(), where, &list)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}

	reply.List = ZoneTree(list)

	return reply, nil

}

